#include "chess.h"
/* global variables */
cell board[MAX_TABLE_SIZE][MAX_TABLE_SIZE];
FILE *fopen();
char output_file_name[128]="";
char input_file_name[128]="";
char path_to_gifs[128]="../set";
char white_position[MAX_TABLE_SIZE*6+1]="White: ";
char black_position[MAX_TABLE_SIZE*6+1]="Black: ";
char title[128]="";
int chess=1;
int width=8;
int height=8;

void usage(char *arg) {
  fprintf(stderr, "prints chess/checkers diagram\n");
  fprintf(stderr, "usage: %s [-c[heckers]] [-w[idth] ##] [-h[eight] ## [-p[ath_to_gifs] ****]\n]",arg);
  fprintf(stderr, "       [[-i input_file] [-o output_file]] | [input_file [output_file]]\n");
  fprintf(stderr, "Input file format\n");
  fprintf(stderr, "[[[W[hite]:]] Ke1,Qd1,Ra1,Rh1,Bc1,Bf1,Nb1,Ng1\n");
  fprintf(stderr, ";\n");
  fprintf(stderr, "[[[B[lack]:]] Ke8,Qd8,Ra8,Rh8,Bc8,Bf8,Nb8,Ng8,\n");
  fprintf(stderr, "a7,b7,c7,d7,e7,f7,g7,h7\n");
  fprintf(stderr, "\n");
  fprintf(stderr, "\",\" separates pieces, \";\" separates sides\n");
  fprintf(stderr, "under -checkers recognises Dc3 also\n");
  exit(100);
}

int position (char *token, int *column, int *row) {
  int tmp;
  if (chess&&(width<10)&&(height<10)&&sscanf(token,"%d",&tmp))
    if (((*column=tmp/10)<=width)&&((*row=tmp%10)<height))
      if ((*column)&&(*row)) return (1);
  if (!chess&&(width==10)&&(height==10)&&sscanf(token,"%d",&tmp)) {
    fprintf( stderr, "internationall chess not implemented yet");
    exit(666);
  }
  if(isalpha(token[0])) {
    *column=toupper(token[0])-'@';
    token++;
    if (sscanf(token,"%d",row)) return (1);
  }
  return(0);
}

void process_token(int side, char *token) {
  int column;
  int row;
  char *tmp;

  if (!*token) return;
  if (side==TITLE) strcat(title,token);
  if (position(token,&column,&row)) {
    if (chess) (board[row-1][column-1]).piece=PAWN;
    else    (board[row-1][column-1]).piece=SIMPLE;
    (board[row-1][column-1]).side=side;
    return;
  }
  tmp=token;
  switch (toupper(*(tmp++))) {
  case 'W': return; /* white */
  case 'B': if ((*tmp==0)||(toupper(*tmp=='L')&&
                            ((tmp[1]==0)||(toupper(tmp[1])=='A'&&
                             ((tmp[2]==0)||(toupper(tmp[2])=='C'&&
                              ((tmp[3]==0)||(toupper(tmp[1])=='K'&&
                               ((tmp[4]==0)))))))))) return; /* black */
  else if (position(tmp,&column,&row)) {
    (board[row-1][column-1]).piece=BISHOP;
    (board[row-1][column-1]).side=side;
    break;
  }
  return;
  case 'D': if (position(tmp,&column,&row)) {
    (board[row-1][column-1]).piece=DAME;
    (board[row-1][column-1]).side=side;
    break;
  }
  return;
  case 'K': if (position(tmp,&column,&row)) {
    (board[row-1][column-1]).piece=KING;
    (board[row-1][column-1]).side=side;
    break;
  }
  return;
  case 'N': if (position(tmp,&column,&row)) {
    (board[row-1][column-1]).piece=KNIGHT;
    (board[row-1][column-1]).side=side;
    break;
  }
  return;
  case 'P': if (position(tmp,&column,&row)) {
    (board[row-1][column-1]).piece=PAWN;
    (board[row-1][column-1]).side=side;
    break;
  }
  return;
  case 'Q': if (position(tmp,&column,&row)) {
    (board[row-1][column-1]).piece=QUEEN;
    (board[row-1][column-1]).side=side;
    break;
  }
  return;
  case 'R': if (position(tmp,&column,&row)) {
    (board[row-1][column-1]).piece=ROOK;
    (board[row-1][column-1]).side=side;
    break;
  }
  return;
  case 'S': if (position(tmp,&column,&row)) {
    (board[row-1][column-1]).piece=SIMPLE;
    (board[row-1][column-1]).side=side;
    break;
  }
  return;
  default: break;
  }
  return;
}

void read_position(void){
  FILE *finp;
  int column;
  int row;
  int side;
  char token[128];
  char *tmp;
  if (strcmp(input_file_name,"")) {
    if ((finp = fopen(input_file_name, "r")) == NULL) {
      fprintf(stderr, "Can't open %s for reading\n",input_file_name);
      exit(2);
    };
  } else {
    finp=stdin;
  };
  for(column=0;column<width;column++)
    for(row=0;row<height;row++) {
      (board[row][column]).piece=EMPTY;
      (board[row][column]).field_colour=(row+column)%2;
      (board[row][column]).side=(board[row][column]).field_colour;
    }
  side=WHITE;
  tmp=token;
  while ((*tmp=getc(finp))!=EOF) {
    switch (*(tmp++)) {
    case ';': {
      if (side!=TITLE) {
        *tmp=0;
        process_token(side,token);
        tmp=token;
      }
      if (side==BLACK) side=TITLE;
      if (side==WHITE) side=BLACK;
      break;
    }
    case '\n': if (side==TITLE) {
      tmp--;
      while ((*tmp==' ')||(*tmp=='\t'||(*tmp=='\b'))) *(tmp--)=0;
      *tmp=0;
      if (strlen(token)) {
        strcat(tmp,"<BR>\n");
        tmp=&tmp[5];
      }
    }
    case '\r':
    case '\f': if (side!=TITLE) tmp--;
    case ':':
    case ',':
      if (side!=TITLE) {
        *tmp=0;
        process_token(side,token);
        tmp=token;
        break;
      }
    case ' ':
    case '\b':
    case '\t': if (side!=TITLE) tmp--;
    default: break;
    }
  }
  *tmp=0;
  process_token(side,token);
}

void check_board(void){
  int column;
  int row;
  char tmp[256];
  int counter[2]={0,0};
  char *position[2]={black_position,white_position};

  for(row=0;row<MAX_TABLE_SIZE*6;row++)
    white_position[row]=black_position[row]=0;
  strcpy(position[WHITE], "White: ");
  strcpy(position[BLACK], "Black: ");
  for(row=0;row<height;row++) for(column=0;column<height;column++)
    if((board[row][column]).piece==KING) {
      counter[(board[row][column]).side]++;
      sprintf(tmp,"K%c%d, ", column+'a', row+1);
      strcat(position[(board[row][column]).side],tmp);
    }
  for(row=0;row<height;row++) for(column=0;column<height;column++)
    if((board[row][column]).piece==QUEEN) {
      counter[(board[row][column]).side]++;
      sprintf(tmp,"Q%c%d, ", column+'a', row+1);
      strcat(position[(board[row][column]).side],tmp);
    }
  for(row=0;row<height;row++) for(column=0;column<height;column++)
    if((board[row][column]).piece==ROOK) {
      counter[(board[row][column]).side]++;
      sprintf(tmp,"R%c%d, ", column+'a', row+1);
      strcat(position[(board[row][column]).side],tmp);
    }
  for(row=0;row<height;row++) for(column=0;column<height;column++)
    if((board[row][column]).piece==BISHOP) {
      counter[(board[row][column]).side]++;
      sprintf(tmp,"K%c%d, ", column+'a', row+1);
      strcat(position[(board[row][column]).side],tmp);
    }
  for(row=0;row<height;row++) for(column=0;column<height;column++)
    if((board[row][column]).piece==KNIGHT) {
      counter[(board[row][column]).side]++;
      sprintf(tmp,"N%c%d, ", column+'a', row+1);
      strcat(position[(board[row][column]).side],tmp);
    }
  for(row=0;row<height;row++) for(column=0;column<height;column++)
    if((board[row][column]).piece==DAME) {
      counter[(board[row][column]).side]++;
      sprintf(tmp,"D%c%d, ", column+'a', row+1);
      strcat(position[(board[row][column]).side],tmp);
    }
  for(row=0;row<height;row++) for(column=0;column<height;column++)
    if((board[row][column]).piece==SIMPLE) {
      counter[(board[row][column]).side]++;
      sprintf(tmp,"%c%d, ", column+'a', row+1);
      strcat(position[(board[row][column]).side],tmp);
    }
  for(row=0;row<height;row++) for(column=0;column<height;column++)
    if((board[row][column]).piece==PAWN) {
      counter[(board[row][column]).side]++;
      sprintf(tmp,"%c%d, ", column+'a', row+1);
      strcat(position[(board[row][column]).side],tmp);
    }
  position[WHITE][strlen(position[WHITE])-2]=':';
  position[BLACK][strlen(position[BLACK])-2]=':';
  sprintf(tmp,"(%d)",counter[WHITE]);
  strcat(position[WHITE],tmp);
  sprintf(tmp,"(%d)",counter[BLACK]);
  strcat(position[BLACK],tmp);
}

void print_diagram(void){
  /* array of names, corresponding to [piece][side][field_colour] */
  static unsigned char code_table[][2][2][8] =
  {{{"b.gif", "b.gif"}, {"w.gif", "w.gif"}},
   {{"bbb.gif", "wbb.gif"}, {"bbw.gif", "wbw.gif"}},
   {{"bdb.gif", "wdb.gif"}, {"bdw.gif", "wdw.gif"}},
   {{"bkb.gif", "wkb.gif"}, {"bkw.gif", "wkw.gif"}},
   {{"bnb.gif", "wnb.gif"}, {"bnw.gif", "wnw.gif"}},
   {{"bpb.gif", "wpb.gif"}, {"bpw.gif", "wpw.gif"}},
   {{"bqb.gif", "wqb.gif"}, {"bqw.gif", "wqw.gif"}},
   {{"brb.gif", "wrb.gif"}, {"brw.gif", "wrw.gif"}},
   {{"bsb.gif", "wsb.gif"}, {"bsw.gif", "wsw.gif"}}};
  FILE *fout;
  int column;
  int row;
  if (strcmp(output_file_name,"")) {
    if ((fout = fopen(output_file_name, "r")) == NULL) {
      fprintf(stderr, "Can't find %s; use cheshead to create\n",output_file_name);
      exit(2);
    };
    fclose(fout);
    if ((fout = fopen(output_file_name, "a")) == NULL) {
      fprintf(stderr, "Can't open %s for writing\n",output_file_name);
      exit(2);
    };
  } else {
    fout=stdout;
  };
  fprintf(fout, "<P>\n<TABLE ALIGN=CENTER BORDER=5 CELLSPACING=0 CELLPADDING=0>\n");
  fprintf(fout,"<TR><TD></TD>\n");
  for(column=0;column<width;column++)
    fprintf(fout,"<TD ALIGN=CENTER>%c</TD>\n",column+'A');
  fprintf(fout,"<TD></TD></TR>\n");
  for(row=height;row-->0;) {
    fprintf(fout,"<TR><TD>%d</TD>\n",row+1);
    for(column=0;column<width;column++)
      fprintf(fout,"<TD><IMG SRC=\"%s/%s\"></TD>\n",path_to_gifs,
              code_table[(board[row][column]).piece]
              [(board[row][column]).field_colour]
              [(board[row][column]).side]);
    fprintf(fout,"<TD>%d</TD></TR>\n",row+1);
  }
  fprintf(fout,"<TR><TD></TD>\n");
  for(column=0;column<width;column++)
    fprintf(fout,"<TD ALIGN=CENTER>%c</TD>\n",column+'A');
  fprintf(fout,"<TD></TD></TR>\n");
  while (title[strlen(title)-1]==' ') title[strlen(title)-1]=0;
  if (!*title) strcpy(title,"diagram &#35; ");
  fprintf(fout,"<CAPTION ALIGN=BOTTOM>%s</CAPTION></TABLE>\n",title);
  fprintf(fout,"<P>\n%s<BR>\n%s<BR>\n<P>\n",white_position,black_position);
  fclose(fout);
  return;
}

int main(int argc, char *argv[])
{
  int i;
  int waiting=0;
  int inputfiledefined=0;
  char *tmp;

  for (i=1;i<argc;i++) {
    tmp=argv[i];
    switch (waiting) {
    case 0: if(tmp[0]=='-') {
      switch (toupper(tmp[1])) {
      case 'C': chess=CHECKERS; break;
      case 'I': waiting=INPUT; break;
      case 'H': waiting=HEIGHT; break;
      case 'O': waiting=OUTPUT; break;
      case 'P': waiting=PATH; break;
      case 'W': waiting=WIDTH; break;
      default: usage(argv[0]);break;
      }; /* switch tmp[0] */
    } else {
      if(0==inputfiledefined++) strcpy(input_file_name,argv[i]);
      else if(2==inputfiledefined) strcpy(output_file_name,argv[i]);
      else if(3<=inputfiledefined) usage(argv[0]);
    };
    break;
    case INPUT: inputfiledefined=3;
      strcpy(input_file_name,argv[i]);
      waiting=0;
      break;
    case OUTPUT: inputfiledefined=3;
      strcpy(output_file_name,argv[i]);
      waiting=0;
      break;
    case WIDTH: sscanf(argv[i],"%d",&width);
      if(!width) usage(argv[0]);
      waiting=0;
      break;
    case HEIGHT: sscanf(argv[i],"%d",&height);
      if(!height) usage(argv[0]);
      waiting=0;
      break;
    case PATH: strcpy(path_to_gifs,argv[i]);
      waiting=0;
      break;
    default: waiting=0; break;
    } /* switch waiting */
  } /* for */
  if (waiting) usage(argv[0]);
  read_position();
  check_board();
  print_diagram();
  return 0;
} /* main */
