#include "chess.h"
/* global variables */
FILE *fopen(),*fout;
char output_file_name[128]="";
char input_file_name[128]="";
int side=WHITE;
char title[128]="";
int row=1;

void usage(char *arg) {
  fprintf(stderr, "prints a seria of chess/checkers moves to HTML file\n");
  fprintf(stderr, "usage: %s [[-i input_file] [-o output_file]]]|[input_file [output_file]] -t title\n", arg);
  fprintf(stderr, "       Input file format\n");
  fprintf(stderr, "       e2-e4 e7-e5\n");
  fprintf(stderr, "          ....\n");
  fprintf(stderr, "the newlines and whitespaces are separators\n");
  exit(100);
}

void process_token(char *token) {
  if (!*token) return;
  if (side==WHITE) {
    fprintf(fout,"<TR><TD ALIGN=RIGHT>%d.</TD><TD></TD><TD ALIGN=LEFT>%s</TD>",row++,token);
    side=BLACK;
  } else {
    fprintf(fout,"<TD></TD><TD ALIGN=LEFT>%s</TD></TR>\n",token);
    side=WHITE;
  }
  return;
}

void read_position(void){
  FILE *finp;
  char token[128];
  char *tmp;
  if (strcmp(input_file_name,"")) {
    if ((finp = fopen(input_file_name, "r")) == NULL) {
      fprintf(stderr, "Can't open %s for reading\n",input_file_name);
      exit(2);
    };
  } else {
    finp=stdin;
  };
  side=WHITE;
  tmp=token;
  while ((*tmp=getc(finp))!=EOF) {
    switch (*(tmp++)) {
    case '\n': 
    case '\r':
    case '\f': 
    case '\b':
    case '\t': tmp[-1]=' ';
    case ' ': *tmp=0; process_token(token);tmp=token;
    default: break;
    }
  }
  *tmp=0;
  process_token(token);
}

int main(int argc, char *argv[])
{
  int i;
  int waiting=0;
  int inputfiledefined=0;
  char *tmp;

  for (i=1;i<argc;i++) {
    tmp=argv[i];
    switch (waiting) {
    case 0: if(tmp[0]=='-') {
      switch (toupper(tmp[1])) {
      case 'I': waiting=INPUT; break;
      case 'O': waiting=OUTPUT; break;
      case 'T': waiting=TITLE; break;
      default: usage(argv[0]);break;
      }; /* switch tmp[0] */
    } else {
      if (0==inputfiledefined++) strcpy (input_file_name,argv[i]);
      else if (2==inputfiledefined) strcpy(output_file_name,argv[i]);
      else if (3<=inputfiledefined) usage(argv[0]);
    };
    break;
    case INPUT: inputfiledefined=3;
      strcpy(input_file_name,argv[i]);
      waiting=0;
      break;
    case OUTPUT: inputfiledefined=3;
      strcpy(output_file_name,argv[i]);
      waiting=0;
      break;
    case TITLE: strcpy(title,argv[i]);
      waiting=0;
      break;
    default: waiting=0; break;
    } /* switch waiting */
  } /* for */
  if (waiting) usage(argv[0]);
  if (strcmp(output_file_name,"")) {
    if ((fout = fopen(output_file_name, "r")) == NULL) {
      fprintf(stderr, "Can't find %s; use cheshead to create\n",output_file_name);
      exit(2);
    };
    fclose(fout);
    if ((fout = fopen(output_file_name, "a")) == NULL) {
      fprintf(stderr, "Can't open %s for writing\n",output_file_name);
      exit(2);
    };
  } else {
    fout=stdout;
  };
  fprintf(fout,"<TABLE ALIGN=LEFT BORDER=0 CELLSPACING=0 CELLPADDING=0>\n");
  fprintf(fout,"<TR><TD></TD><TD ALIGN=RIGHT WIDTH=20em></TD><TD></TD>\
<TD ALIGN=LEFT WIDTH=20em></TD><TD></TD><TD ALIGN=RIGHT WIDTH=20em></TD>\
<TD></TD><TD ALIGN=RIGHT WIDTH=20em></TD></TR>\n\n");
  read_position();
  if (side==BLACK) process_token("");
  fprintf(fout,"<CAPTION ALIGN=BOTTOM>\n</CAPTION></TABLE>\n");
  fclose(fout);
  return 0;
} /* main */

